@@ -86,22 +86,31 @@ import pathlib
 import tempfile
 import uuid
 import zipfile
+from contextlib import ExitStack
 from typing import Any
 
 import polars as pl
 import pyarrow.parquet as pq
 import tacozip
 
-from tacotoolbox._column_utils import reorder_internal_columns, cast_dataframe_to_schema
+from tacotoolbox._column_utils import (
+    align_dataframe_schemas,
+    cast_dataframe_to_schema,
+    reorder_internal_columns,
+)
 from tacotoolbox._constants import (
     METADATA_OFFSET,
     METADATA_PARENT_ID,
     METADATA_SIZE,
     is_padding_id,
 )
+from tacotoolbox._logging import get_logger
 from tacotoolbox._metadata import MetadataPackage
+from tacotoolbox._progress import ProgressContext
 from tacotoolbox._virtual_zip import VirtualTACOZIP
 
+logger = get_logger(__name__)
+
 
 class ZipWriterError(Exception):
     """Raised when ZIP writing operations fail."""
@@ -112,21 +121,17 @@ class ZipWriter:
     Handle creation of .tacozip container files with precalculated offsets.
 
     The ZipWriter uses a sophisticated bottom-up approach to ensure all
-    metadata files (__meta__) have correct offsets pointing to data files:
+    metadata files (__meta__) have correct offsets pointing to data files.
 
-    1. Virtual ZIP stage: Calculate all file offsets without writing
-    2. Bottom-up metadata: Generate __meta__ starting from deepest folders
-    3. Offset propagation: Each level's offsets affect parent level metadata
-    4. Final assembly: Write actual ZIP with all offsets correct
-
-    This ensures that FOLDER samples in metadata point to their __meta__ files,
-    and FILE samples point to their actual data, all with byte-perfect offsets.
+    All temporary files are automatically cleaned up via ExitStack, even
+    if errors occur during ZIP creation.
     """
 
     def __init__(
         self,
         output_path: pathlib.Path,
-        quiet: bool = True,
+        quiet: bool = False,
+        debug: bool = False,
         temp_dir: pathlib.Path | None = None,
     ) -> None:
         """
@@ -134,7 +139,8 @@ class ZipWriter:
 
         Args:
             output_path: Path for output .tacozip file
-            quiet: If True, suppress progress messages (default: True)
+            quiet: If True, hide progress bars (default: False)
+            debug: If True, enable debug logging (default: False)
             temp_dir: Directory for temporary files (default: system temp)
         """
         self.output_path = output_path
@@ -146,7 +152,13 @@ class ZipWriter:
             self.temp_dir = pathlib.Path(temp_dir)
 
         self.temp_dir.mkdir(parents=True, exist_ok=True)
-        self._temp_files: list[pathlib.Path] = []
+
+        # Use ExitStack for robust cleanup
+        self._cleanup_stack = ExitStack()
+
+        logger.debug(
+            f"ZipWriter initialized: output={output_path}, temp_dir={self.temp_dir}"
+        )
 
     def create_complete_zip(
         self,
@@ -165,74 +177,63 @@ class ZipWriter:
             src_files: Source file paths (filesystem)
             arc_files: Archive paths in ZIP (e.g., "DATA/sample.tif")
             metadata_package: Complete metadata from MetadataGenerator
-            **parquet_kwargs: Additional arguments for Parquet writer
+            **parquet_kwargs: Additional Parquet writer parameters
+                            - For local __meta__: simple compression params
+                            - For consolidated metadata: merged with PARQUET_CDC_DEFAULT_CONFIG
+                            Example: compression_level=22
 
         Returns:
-            Path to created .tacozip file
+            pathlib.Path: Path to created .tacozip file
 
         Raises:
             ZipWriterError: If ZIP creation fails
+            
+        Example:
+            >>> writer.create_complete_zip(
+            ...     src_files, arc_files, metadata_package,
+            ...     compression_level=22  # Override default compression
+            ... )
         """
         try:
-            if not self.quiet:
-                print("=" * 60)
-                print("STARTING BOTTOM-UP __meta__ GENERATION")
-                print("=" * 60)
-
-            if not self.quiet:
-                print("\n[STEP 1] Adding data files to VirtualZIP...")
+            logger.info("Starting bottom-up __meta__ generation")
 
+            # Step 1: Add data files to VirtualZIP
             virtual_zip = VirtualTACOZIP()
             num_entries = len(metadata_package.levels) + 1
             virtual_zip.add_header()
 
+            logger.debug(f"Adding {len(src_files)} data files to virtual ZIP")
             for src_path, arc_path in zip(src_files, arc_files, strict=False):
                 if pathlib.Path(src_path).exists():
                     virtual_zip.add_file(src_path, arc_path)
 
-            if not self.quiet:
-                print(f"  Added {len(src_files)} data files")
-
-            if not self.quiet:
-                print("\n[STEP 2] Calculating initial offsets for data files...")
-
+            # Step 2: Calculate initial offsets
             virtual_zip.calculate_offsets()
             offsets_map = virtual_zip.get_all_offsets()
+            logger.debug(f"Initial offsets calculated: {len(offsets_map)} entries")
 
-            if not self.quiet:
-                print(f"  Initial offsets calculated: {len(offsets_map)} entries")
-
-            if not self.quiet:
-                print("\n[STEP 3] Analyzing folder hierarchy...")
-
+            # Step 3: Analyze folder hierarchy
             folder_order = self._extract_folder_order(arc_files)
             folders_by_depth = self._group_folders_by_depth(folder_order)
 
-            if not self.quiet:
-                for depth in sorted(folders_by_depth.keys(), reverse=True):
-                    print(f"  Depth {depth}: {len(folders_by_depth[depth])} folders")
+            for depth in sorted(folders_by_depth.keys(), reverse=True):
+                logger.debug(f"Depth {depth}: {len(folders_by_depth[depth])} folders")
 
-            if not self.quiet:
-                print("\n[STEP 4] Generating __meta__ files (bottom-up)...")
-
-            temp_parquet_meta_files = {}
+            # Step 4: Generate __meta__ files (bottom-up)
+            temp_meta_files = {}
             enriched_metadata_by_depth = {}
             max_depth = max(folders_by_depth.keys()) if folders_by_depth else 0
 
             # Bottom-up: Start from deepest folders
             for depth in range(max_depth, 0, -1):
-                if not self.quiet:
-                    print(f"\n  Processing depth {depth}...")
+                logger.debug(f"Processing depth {depth}...")
 
                 folders_at_depth = folders_by_depth[depth]
                 enriched_metadata_by_depth[depth] = []
 
                 for folder_path in folders_at_depth:
                     if folder_path not in metadata_package.local_metadata:
-                        if not self.quiet:
-                            print(
-                                f"    WARNING: {folder_path} not in local_metadata, skipping"
-                            )
+                        logger.warning(f"{folder_path} not in local_metadata")
                         continue
 
                     local_df = metadata_package.local_metadata[folder_path]
@@ -243,8 +244,10 @@ class ZipWriter:
 
                     # Write __meta__ as Parquet
                     meta_arc_path = f"{folder_path}__meta__"
-                    temp_parquet = self._write_single_parquet(enriched_df, folder_path)
-                    temp_parquet_meta_files[meta_arc_path] = temp_parquet
+                    temp_parquet = self._write_single_parquet(
+                        enriched_df, folder_path, **parquet_kwargs
+                    )
+                    temp_meta_files[meta_arc_path] = temp_parquet
 
                     # Add to virtual ZIP and recalculate offsets
                     real_size = temp_parquet.stat().st_size
@@ -252,20 +255,13 @@ class ZipWriter:
                         str(temp_parquet), meta_arc_path, file_size=real_size
                     )
 
-                    if not self.quiet:
-                        print(f"    Created {meta_arc_path} ({real_size} bytes)")
-
                 # Recalculate offsets after adding __meta__ files
                 virtual_zip.calculate_offsets()
                 offsets_map = virtual_zip.get_all_offsets()
 
-                if not self.quiet:
-                    print(f"  Recalculated offsets: {len(offsets_map)} entries")
+            logger.debug("Rebuilding consolidated metadata...")
 
-            if not self.quiet:
-                print(
-                    "\n[STEP 5] Rebuilding consolidated metadata (METADATA/levelX.parquet)..."
-                )
+            # Step 5: Rebuild consolidated metadata (METADATA/levelX.parquet)
 
             # Level 0: Add offset/size columns
             original_level0 = metadata_package.levels[0]
@@ -284,26 +280,20 @@ class ZipWriter:
                 metadata_package.levels[0] = pl.concat(
                     [original_level0, offset_size_df], how="horizontal"
                 )
-                # Reorder to place internal:* columns at the end
                 metadata_package.levels[0] = reorder_internal_columns(
                     metadata_package.levels[0]
                 )
             else:
                 metadata_package.levels[0] = original_level0
 
-            if not self.quiet:
-                cols_info = metadata_package.levels[0].columns
-                print(
-                    f"  Level 0: {len(metadata_package.levels[0])} samples (columns: {len(cols_info)})"
-                )
+            logger.debug(f"Level 0: {len(metadata_package.levels[0])} samples")
 
             # Level 1+: Preserve parent_id from original, add offsets from concatenated
             for depth in range(1, len(metadata_package.levels)):
                 if enriched_metadata_by_depth.get(depth):
-                    # Original DataFrame (HAS parent_id, NO offset/size)
                     original_level = metadata_package.levels[depth]
 
-                    # Extract field schema for this level from collection
+                    # Extract field schema for this level
                     field_schema = metadata_package.collection.get(
                         "taco:field_schema", {}
                     )
@@ -317,8 +307,12 @@ class ZipWriter:
                             for df in enriched_dfs
                         ]
 
-                    # Concatenated DataFrame (HAS offset/size, NO parent_id)
-                    concatenated = pl.concat(enriched_dfs, how="vertical")
+                    # Align schemas before concatenating
+                    # Different folders may have children with different extensions
+                    # (e.g., folder_A has [cloudmask, s2data], folder_B has [cloudmask, thumbnail])
+                    # Without this, pl.concat fails with: ShapeError: unable to append DataFrame
+                    aligned_dfs = align_dataframe_schemas(enriched_dfs)
+                    concatenated = pl.concat(aligned_dfs, how="vertical")
 
                     # Extract only offset/size columns
                     offset_size_cols = [METADATA_OFFSET, METADATA_SIZE]
@@ -347,36 +341,31 @@ class ZipWriter:
                     else:
                         metadata_package.levels[depth] = original_level
 
-                    if not self.quiet:
-                        cols_info = metadata_package.levels[depth].columns
-                        has_parent_id = METADATA_PARENT_ID in cols_info
-                        print(
-                            f"  Level {depth}: {len(metadata_package.levels[depth])} samples "
-                            f"(rebuilt from {len(enriched_dfs)} folders, "
-                            f"parent_id={has_parent_id})"
-                        )
-                else:
-                    if not self.quiet:
-                        print(f"  Level {depth}: No data (skipped)")
+                    logger.debug(
+                        f"Level {depth}: {len(metadata_package.levels[depth])} samples "
+                        f"(parent_id={METADATA_PARENT_ID in metadata_package.levels[depth].columns})"
+                    )
 
             # Write consolidated METADATA/levelX.parquet files
-            temp_parquet_level_files = {}
+            temp_level_files = {}
             for i, level_df in enumerate(metadata_package.levels):
                 arc_path = f"METADATA/level{i}.parquet"
                 temp_path = self.temp_dir / f"{uuid.uuid4().hex}_level{i}.parquet"
                 arrow_table = level_df.to_arrow()
-                pq.write_table(arrow_table, temp_path, **parquet_kwargs)
+                
+                # Merge CDC defaults with user kwargs
+                from tacotoolbox._constants import PARQUET_CDC_DEFAULT_CONFIG
+                parquet_config = {**PARQUET_CDC_DEFAULT_CONFIG, **parquet_kwargs}
+                
+                pq.write_table(arrow_table, temp_path, **parquet_config)
                 real_size = temp_path.stat().st_size
                 virtual_zip.add_file(str(temp_path), arc_path, file_size=real_size)
-                temp_parquet_level_files[arc_path] = temp_path
-                self._temp_files.append(temp_path)
+                temp_level_files[arc_path] = temp_path
+                self._register_temp_file(temp_path)
 
-                if not self.quiet:
-                    print(f"  Added {arc_path} ({real_size} bytes)")
-
-            if not self.quiet:
-                print("\n[STEP 6] Adding COLLECTION.json...")
+                logger.debug(f"Added {arc_path} ({real_size} bytes)")
 
+            # Add COLLECTION.json
             collection = metadata_package.collection.copy()
             collection["taco:pit_schema"] = metadata_package.pit_schema
             collection["taco:field_schema"] = metadata_package.field_schema
@@ -387,18 +376,12 @@ class ZipWriter:
             virtual_zip.add_file(
                 str(temp_json), "COLLECTION.json", file_size=collection_size
             )
-            self._temp_files.append(temp_json)
-
-            if not self.quiet:
-                print(f"  Added COLLECTION.json ({collection_size} bytes)")
-
-            if not self.quiet:
-                print("\n[STEP 7] Final offset calculation...")
+            self._register_temp_file(temp_json)
 
+            # Final offset calculation
             virtual_zip.calculate_offsets()
 
-            if not self.quiet:
-                print("\n[STEP 8] Preparing final file lists for ZIP creation...")
+            logger.debug("Preparing final file lists for ZIP creation")
 
             all_src_files = list(src_files)
             all_arc_files = list(arc_files)
@@ -407,15 +390,15 @@ class ZipWriter:
             for depth in range(max_depth, 0, -1):
                 for folder_path in folders_by_depth[depth]:
                     meta_arc_path = f"{folder_path}__meta__"
-                    if meta_arc_path in temp_parquet_meta_files:
-                        temp_path = temp_parquet_meta_files[meta_arc_path]
+                    if meta_arc_path in temp_meta_files:
+                        temp_path = temp_meta_files[meta_arc_path]
                         all_src_files.append(str(temp_path))
                         all_arc_files.append(meta_arc_path)
 
             # Add METADATA/levelX.parquet files
             for i in range(len(metadata_package.levels)):
                 arc_path = f"METADATA/level{i}.parquet"
-                temp_path = temp_parquet_level_files[arc_path]
+                temp_path = temp_level_files[arc_path]
                 all_src_files.append(str(temp_path))
                 all_arc_files.append(arc_path)
 
@@ -423,24 +406,20 @@ class ZipWriter:
             all_src_files.append(str(temp_json))
             all_arc_files.append("COLLECTION.json")
 
-            if not self.quiet:
-                print(f"  Total files in ZIP: {len(all_src_files)}")
-
-            if not self.quiet:
-                print("\n[STEP 9] Writing final ZIP file...")
+            logger.info(f"Writing ZIP with {len(all_src_files)} total files")
 
+            # Write ZIP with placeholder header
             header_entries = [(0, 0) for _ in range(num_entries)]
 
-            tacozip.create(
-                zip_path=str(self.output_path),
-                src_files=all_src_files,
-                arc_files=all_arc_files,
-                entries=header_entries,
-            )
-
-            if not self.quiet:
-                print("\n[STEP 10] Updating TACO_HEADER with real offsets...")
+            with ProgressContext(quiet=self.quiet):
+                tacozip.create(
+                    zip_path=str(self.output_path),
+                    src_files=all_src_files,
+                    arc_files=all_arc_files,
+                    entries=header_entries,
+                )
 
+            # Update TACO_HEADER with real offsets
             metadata_offsets, metadata_sizes = self._get_metadata_offsets()
             collection_offset, collection_size = self._get_collection_offset()
 
@@ -451,21 +430,27 @@ class ZipWriter:
 
             tacozip.update_header(zip_path=str(self.output_path), entries=real_entries)
 
-            if not self.quiet:
-                print(f"\n{'='*60}")
-                print(f"ZIP CREATED SUCCESSFULLY: {self.output_path}")
-                print(f"{'='*60}\n")
+            logger.info(f"ZIP created successfully: {self.output_path}")
+            return self.output_path
 
         except Exception as e:
+            logger.error(f"Failed to create ZIP: {e}")
             raise ZipWriterError(f"Failed to create ZIP: {e}") from e
-        else:
-            return self.output_path
         finally:
             # CRITICAL: Always cleanup temporary files
-            if not self.quiet:
-                print("\n[CLEANUP] Removing temporary files...")
             self._cleanup()
 
+    def _register_temp_file(self, path: pathlib.Path) -> None:
+        """
+        Register temporary file for automatic cleanup.
+
+        Uses ExitStack to ensure cleanup even if exceptions occur.
+
+        Args:
+            path: Path to temporary file
+        """
+        self._cleanup_stack.callback(path.unlink, missing_ok=True)
+
     def _extract_folder_order(self, arc_files: list[str]) -> list[str]:
         """
         Extract folder paths that need __meta__ files.
@@ -473,10 +458,10 @@ class ZipWriter:
         Only includes Level 1+ folders (not DATA/ root).
 
         Args:
-            arc_files: Archive paths of data files
+            arc_files: List of archive file paths
 
         Returns:
-            List of folder paths (e.g., ["DATA/folder_A/", "DATA/folder_A/sub/"])
+            list[str]: Sorted list of folder paths needing __meta__
         """
         folder_set = set()
 
@@ -498,12 +483,7 @@ class ZipWriter:
             folder_order: List of folder paths
 
         Returns:
-            Dictionary mapping depth -> list of folder paths
-
-        Example:
-            >>> folders = ["DATA/a/", "DATA/a/b/", "DATA/a/b/c/"]
-            >>> self._group_folders_by_depth(folders)
-            {1: ["DATA/a/"], 2: ["DATA/a/b/"], 3: ["DATA/a/b/c/"]}
+            dict[int, list[str]]: Folders grouped by depth level
         """
         by_depth: dict[int, list[str]] = {}
 
@@ -516,38 +496,47 @@ class ZipWriter:
 
         return by_depth
 
-    def _write_single_parquet(self, df: pl.DataFrame, folder_path: str) -> pathlib.Path:
+    def _write_single_parquet(
+        self, 
+        df: pl.DataFrame, 
+        folder_path: str,
+        **parquet_kwargs: Any
+    ) -> pathlib.Path:
         """
         Write a single __meta__ Parquet file to temp directory.
 
         Args:
-            df: DataFrame with metadata
-            folder_path: Folder path for identifier
+            df: DataFrame to write
+            folder_path: Folder path (used for filename)
+            **parquet_kwargs: Additional Parquet writer parameters
 
         Returns:
-            Path to temporary Parquet file
+            pathlib.Path: Path to created temp file
         """
         identifier = folder_path.replace("/", "_").strip("_")
         temp_path = self.temp_dir / f"{uuid.uuid4().hex}_{identifier}.parquet"
 
         filtered_df = self._filter_metadata_columns(df)
         arrow_table = filtered_df.to_arrow()
-        pq.write_table(arrow_table, temp_path, compression="zstd")
+        
+        # Default config for local __meta__ (simple, no CDC)
+        default_config = {"compression": "zstd"}
+        parquet_config = {**default_config, **parquet_kwargs}
+        
+        pq.write_table(arrow_table, temp_path, **parquet_config)
 
-        self._temp_files.append(temp_path)
+        self._register_temp_file(temp_path)
         return temp_path
 
     def _filter_metadata_columns(self, df: pl.DataFrame) -> pl.DataFrame:
         """
-        Filter columns for __meta__ files.
-
-        Removes container-irrelevant columns like 'path'.
+        Filter columns for __meta__ files (removes 'path' column).
 
         Args:
-            df: DataFrame with all columns
+            df: DataFrame to filter
 
         Returns:
-            DataFrame with filtered columns
+            pl.DataFrame: Filtered DataFrame
         """
         exclude_columns = {"path"}
         cols_to_keep = [col for col in df.columns if col not in exclude_columns]
@@ -558,7 +547,7 @@ class ZipWriter:
         Get offsets and sizes for METADATA/levelX.parquet files.
 
         Returns:
-            Tuple of (offsets, sizes) lists
+            tuple[list[int], list[int]]: (offsets, sizes) for metadata files
         """
         offsets = []
         sizes = []
@@ -594,7 +583,7 @@ class ZipWriter:
         Get offset and size for COLLECTION.json.
 
         Returns:
-            Tuple of (offset, size)
+            tuple[int, int]: (offset, size) for COLLECTION.json
         """
         with zipfile.ZipFile(self.output_path, "r") as zf:
             with open(self.output_path, "rb") as f:
@@ -615,29 +604,10 @@ class ZipWriter:
         """
         Clean up all temporary files created during ZIP creation.
 
-        This ensures no temporary files are left behind after the ZIP
-        is successfully created or if an error occurs during creation.
+        Uses ExitStack for robust cleanup that works even when exceptions occur.
         """
-        if not self._temp_files:
-            return
-
-        cleaned = 0
-        failed = 0
-
-        for temp_file in self._temp_files:
-            try:
-                if temp_file.exists():
-                    temp_file.unlink()
-                    cleaned += 1
-            except Exception as e:
-                failed += 1
-                if not self.quiet:
-                    print(f"  Warning: Failed to cleanup {temp_file}: {e}")
-
-        self._temp_files.clear()
-
-        if not self.quiet and (cleaned > 0 or failed > 0):
-            print(f"  Cleanup: {cleaned} files removed, {failed} failed")
+        logger.debug("Cleaning up temporary files")
+        self._cleanup_stack.close()
 
 
 def _add_zip_offsets(
@@ -648,34 +618,22 @@ def _add_zip_offsets(
     """
     Add internal:offset and internal:size columns for ZIP containers.
 
-    This is a ZIP-SPECIFIC function. FOLDER containers don't use offsets.
+    This is ZIP-SPECIFIC. FOLDER containers don't use offsets.
 
     Critical behavior:
     - FILE samples: offset points to actual data file in ZIP
     - FOLDER samples: offset points to child's __meta__ file in ZIP
 
-    This function is ALWAYS strict: missing offsets for non-padding samples
-    indicate a bug in offset calculation and will raise an error.
-
     Args:
         metadata_df: DataFrame with sample metadata
         offsets_map: Dictionary mapping arc_path -> (offset, size)
         folder_path: Current folder path (e.g., "DATA/folder_A/")
 
     Returns:
-        DataFrame with added internal:offset and internal:size columns
+        pl.DataFrame: DataFrame with added internal:offset and internal:size columns
 
     Raises:
         ValueError: If offsets missing for non-padding samples
-
-    Example:
-        >>> offsets_map = {
-        ...     "DATA/file1.tif": (1000, 500),
-        ...     "DATA/folder_A/__meta__": (1500, 200)
-        ... }
-        >>> df = _add_zip_offsets(metadata_df, offsets_map, folder_path="DATA/")
-        >>> df.columns
-        ['id', 'type', 'internal:parent_id', 'internal:offset', 'internal:size']
     """
     offsets = []
     sizes = []
@@ -726,4 +684,4 @@ def _add_zip_offsets(
     )
 
     # Reorder to place internal:* columns at the end
-    return reorder_internal_columns(result_df)
+    return reorder_internal_columns(result_df)
\ No newline at end of file
