@@ -1,5 +1,5 @@
 from tacotoolbox.sample.extensions.istac import ISTAC
-from tacotoolbox.sample.extensions.packing import Packing
+from tacotoolbox.sample.extensions.scaling import Scaling
 from tacotoolbox.sample.extensions.stac import STAC
-from tacotoolbox.sample.extensions.stats import GeotiffStats
+from tacotoolbox.sample.extensions.geotiff_stats import GeotiffStats
 from tacotoolbox.sample.extensions.tacotiff import Header as TACOTIFFHeader
