@@ -6,6 +6,7 @@ for training, validation, and testing workflows.
 
 Key features:
 - Supports FILE and FOLDER asset types
+- Automatic type inference from path (type="auto" default)
 - Dynamic extension system for adding metadata
 - Format validation via validators (TacotiffValidator, etc.)
 - Automatic cleanup of temporary files from bytes
@@ -38,8 +39,10 @@ VALID_KEY_PATTERN = re.compile(r"^[a-zA-Z0-9_]+(?:[:][\w]+)?$")
 class SampleExtension(ABC, pydantic.BaseModel):
     """Abstract base class for Sample extensions that compute metadata."""
 
-    return_none: bool = pydantic.Field(
-        False, description="If True, return None values while preserving schema"
+    schema_only: bool = pydantic.Field(
+        False,
+        description="If True, return None values while preserving schema",
+        validation_alias="return_none"
     )
 
     @abstractmethod
@@ -49,21 +52,13 @@ class SampleExtension(ABC, pydantic.BaseModel):
 
     @abstractmethod
     def _compute(self, sample: "Sample") -> pl.DataFrame:
-        """Actual computation logic - only called when return_none=False."""
+        """Actual computation logic - only called when schema_only=False."""
         pass
 
     def __call__(self, sample: "Sample") -> pl.DataFrame:
-        """
-        Process Sample and return computed metadata.
-
-        Args:
-            sample: Input Sample object
-
-        Returns:
-            pl.DataFrame: Single-row DataFrame with computed metadata
-        """
-        # Check return_none FIRST for performance
-        if self.return_none:
+        """Process Sample and return computed metadata as single-row DataFrame."""
+        # Check schema_only FIRST for performance
+        if self.schema_only:
             schema = self.get_schema()
             none_data = {col_name: [None] for col_name in schema}
             return pl.DataFrame(none_data, schema=schema)
@@ -81,6 +76,13 @@ class Sample(pydantic.BaseModel):
     - FILE: Any file-based format (e.g., GeoTIFF, NetCDF, HDF5, PDF, CSV, Zarr)
     - FOLDER: A nested collection of samples (Tortilla)
 
+    Type inference:
+    By default, type is automatically inferred from path:
+    - Path or bytes → FILE
+    - Tortilla → FOLDER
+
+    You can explicitly specify type for validation, but it's optional.
+
     Format-specific validation is handled by validators (applied via validate_with):
     - Use TacotiffValidator for TACOTIFF format validation
     - Use TacozarrValidator for TACOZARR format validation (future)
@@ -93,44 +95,6 @@ class Sample(pydantic.BaseModel):
 
     Temporary files are automatically cleaned up when the Sample is
     garbage collected or when cleanup() is called explicitly.
-
-    Example:
-        >>> from tacotoolbox import Sample
-        >>> from tacotoolbox.sample.validators import TacotiffValidator
-        >>>
-        >>> # Basic sample
-        >>> sample = Sample(
-        ...     id="soyuntaco",
-        ...     path=Path("/home/lxlx/sentinel2.tif"),
-        ...     type="FILE"
-        ... )
-        >>>
-        >>> # Validate with TACOTIFF format
-        >>> sample.validate_with(TacotiffValidator())
-        >>>
-        >>> # Bytes support with auto cleanup
-        >>> sample = Sample(
-        ...     id="bytesample",
-        ...     path=image_bytes,
-        ...     type="FILE",
-        ...     temp_dir="/data/workspace"
-        ... )
-        >>> # Temporary file created automatically
-        >>> # ... use sample ...
-        >>> sample.cleanup()  # Explicit cleanup
-        >>> # Or let garbage collector do it automatically
-        >>>
-        >>> # Extensions
-        >>> sample.extend_with(stac_obj)
-        >>> sample.extend_with({"s2:mgrs_tile": "T30UYA"})
-        >>> sample.extend_with(scaling_extension)
-        >>>
-        >>> # Nested samples (FOLDER)
-        >>> nested = Sample(
-        ...     id="multitemporal",
-        ...     path=Tortilla(samples=[s1, s2, s3]),
-        ...     type="FOLDER"
-        ... )
     """
 
     # Core attributes
@@ -138,7 +102,9 @@ class Sample(pydantic.BaseModel):
     path: (
         pathlib.Path | Tortilla | bytes
     )  # Location of data (file, container, or bytes)
-    type: AssetType  # Type of geospatial data asset
+    type: Literal["FILE", "FOLDER", "auto"] = (
+        "auto"  # Type of geospatial data asset (auto-inferred by default)
+    )
 
     # Private attribute to store temp files for cleanup
     _temp_files: list[pathlib.Path] = pydantic.PrivateAttr(default_factory=list)
@@ -154,13 +120,7 @@ class Sample(pydantic.BaseModel):
     )
 
     def __init__(self, temp_dir: pathlib.Path | None = None, **data):
-        """
-        Initialize Sample with optional temp_dir for bytes conversion.
-
-        Args:
-            temp_dir: Directory for temporary files (if path is bytes)
-            **data: Sample fields (id, path, type, extensions, etc.)
-        """
+        """Initialize Sample with optional temp_dir for bytes conversion."""
         # Handle temp_dir for bytes conversion without storing it
         if "path" in data and isinstance(data["path"], bytes):
             temp_dir = (
@@ -189,7 +149,7 @@ class Sample(pydantic.BaseModel):
             object.__setattr__(self, "_temp_files", [temp_path])
 
         # Extract extension fields (anything not a core field)
-        core_fields = SHARED_CORE_FIELDS
+        core_fields = SHARED_CORE_FIELDS | {"type"}  # Include 'type' as core
         extension_fields = {k: v for k, v in data.items() if k not in core_fields}
 
         # Initialize with all fields (Pydantic accepts them due to extra="allow")
@@ -209,21 +169,6 @@ class Sample(pydantic.BaseModel):
 
         Padding samples use empty bytes (b"") which creates a 0-byte temporary file.
         This file can be copied to ZIP/FOLDER containers like any other file.
-
-        Args:
-            index: Padding sample index (e.g., 0 for __TACOPAD__0)
-            temp_dir: Optional temp directory (defaults to system temp)
-
-        Returns:
-            Sample with __TACOPAD__ ID and empty temp file
-
-        Example:
-            >>> # Internal use only - called by Tortilla
-            >>> padding = Sample._create_padding(index=0)
-            >>> padding.id
-            '__TACOPAD__0'
-            >>> padding.path.stat().st_size
-            0  # 0-byte file
         """
         # Create temp directory
         temp_dir = (
@@ -256,13 +201,7 @@ class Sample(pydantic.BaseModel):
         """
         Clean up temporary files created from bytes.
 
-        This method can be called explicitly to clean up temporary files
-        immediately instead of waiting for garbage collection.
-
-        Example:
-            >>> sample = Sample(id="test", path=image_bytes, type="FILE")
-            >>> # ... use sample ...
-            >>> sample.cleanup()  # Explicit cleanup
+        Call explicitly to clean up immediately instead of waiting for garbage collection.
         """
         if not self._temp_files:
             return
@@ -278,12 +217,7 @@ class Sample(pydantic.BaseModel):
         self._temp_files.clear()
 
     def __del__(self):
-        """
-        Cleanup on garbage collection.
-
-        Automatically removes temporary files when the Sample object
-        is destroyed by the garbage collector.
-        """
+        """Cleanup on garbage collection."""
         try:
             self.cleanup()
         except Exception:
@@ -300,22 +234,6 @@ class Sample(pydantic.BaseModel):
         - NO slashes (/, \) - breaks file paths in ZIP and FOLDER containers
         - NO colons (:) - invalid on Windows, conflicts with extension namespaces
         - NO double underscore prefix (__) - reserved for __TACOPAD__ system
-
-        Args:
-            v: Sample ID to validate
-
-        Returns:
-            Validated ID
-
-        Raises:
-            ValueError: If ID format is invalid
-
-        Example:
-            >>> Sample(id="sample_001", path=b"", type="FILE")  # OK
-            >>> Sample(id="IMG-2024", path=b"", type="FILE")    # OK
-            >>> Sample(id="file/path", path=b"", type="FILE")   # FAIL - slash
-            >>> Sample(id="img:rgb", path=b"", type="FILE")     # FAIL - colon
-            >>> Sample(id="__private", path=b"", type="FILE")   # FAIL - reserved
         """
         # Check for slashes
         if "/" in v or "\\" in v:
@@ -359,15 +277,36 @@ class Sample(pydantic.BaseModel):
         )
 
     @pydantic.model_validator(mode="after")
-    def global_validation(self):
-        """Cross-field validation ensuring path type matches asset type."""
-        # FOLDER type must have Tortilla path
-        if self.type == "FOLDER" and not isinstance(self.path, Tortilla):
-            raise ValueError("FOLDER type must have a Tortilla instance as path")
+    def infer_and_validate_type(self):
+        """
+        Infer type from path or validate explicit type.
 
-        # FILE type must have pathlib.Path (validated in validate_path)
-        if self.type == "FILE" and isinstance(self.path, Tortilla):
-            raise ValueError("FILE type must have a pathlib.Path, not Tortilla")
+        If type="auto" (default), infers type from path:
+        - Path or bytes → FILE
+        - Tortilla → FOLDER
+
+        If type is explicit ("FILE" or "FOLDER"), validates it matches path type.
+
+        After this validator, self.type is ALWAYS "FILE" or "FOLDER" (never "auto").
+        """
+        # Infer type from path
+        inferred_type: AssetType = (
+            "FOLDER" if isinstance(self.path, Tortilla) else "FILE"
+        )
+
+        if self.type == "auto":
+            # Auto mode - use inferred type
+            self.type = inferred_type
+        else:
+            # Explicit mode - validate consistency
+            if self.type != inferred_type:
+                path_type_str = (
+                    "Tortilla" if isinstance(self.path, Tortilla) else "Path/bytes"
+                )
+                raise ValueError(
+                    f"Type mismatch: specified type='{self.type}' but path type ({path_type_str}) "
+                    f"implies type='{inferred_type}'"
+                )
 
         return self
 
@@ -376,35 +315,13 @@ class Sample(pydantic.BaseModel):
         Validate sample using provided validator.
 
         Validators enforce format-specific requirements (e.g., TACOTIFF, TACOZARR).
-        This method allows applying strict format validation after sample creation.
-
-        Args:
-            validator: SampleValidator instance (e.g., TacotiffValidator())
-
-        Raises:
-            ValidationError: If validation fails
-
-        Example:
-            >>> from tacotoolbox.sample.validators import TacotiffValidator
-            >>>
-            >>> sample = Sample(id="s2", path=Path("data.tif"), type="FILE")
-            >>> sample.validate_with(TacotiffValidator())  # Validates TACOTIFF format
         """
         validator.validate(self)
 
     def extend_with(
         self, extension: Any | dict[str, Any], name: str | None = None
     ) -> None:
-        """
-        Add extension to sample by adding fields directly to the model.
-
-        Args:
-            extension: SampleExtension, Pydantic model, or dictionary to add
-            name: Optional custom namespace (defaults to class name for objects)
-
-        Returns:
-            Sample: Self for method chaining
-        """
+        """Add extension to sample by adding fields directly to the model."""
         # Check if this is a computational SampleExtension
         if callable(extension) and hasattr(extension, "model_dump"):
             self._handle_sample_extension(extension)
@@ -516,17 +433,14 @@ class Sample(pydantic.BaseModel):
 
     def export_metadata(self) -> pl.DataFrame:
         """
-        Export complete Sample metadata as a single-row DataFrame with proper schemas.
-
-        Returns all fields in the model, including core attributes and
-        extension metadata with proper data types preserved.
+        Export complete Sample metadata as single-row DataFrame with proper schemas.
 
         Core fields (id, type, path) always use String type for schema consistency,
         even when path is None (FOLDER samples). This ensures all samples have
         compatible schemas for concatenation.
 
-        Returns:
-            pl.DataFrame: Single-row DataFrame with complete sample metadata
+        The 'type' column always contains "FILE" or "FOLDER" (never "auto")
+        because type inference happens during validation.
         """
         data = self.model_dump()
 
@@ -550,4 +464,4 @@ class Sample(pydantic.BaseModel):
         # Create DataFrame with explicit schema
         df = pl.DataFrame([data], schema=complete_schema)
 
-        return df
+        return df
\ No newline at end of file
