@@ -7,16 +7,8 @@ class Header(SampleExtension):
     """
     Extract TACOTIFF binary header metadata.
 
-    Stores the complete binary header (TacoHeader format) in the taco:header field.
+    Stores complete binary header (TacoHeader format) in taco:header field.
     This header contains all metadata needed for ultra-fast TACOTIFF reading.
-
-    Fields:
-        taco:header (bytes): Binary metadata in TacoHeader format
-
-    Example:
-        >>> sample = Sample(id="img", path=Path("data.tacotiff"), type="FILE")
-        >>> sample.extend_with(TacoHeader())
-        >>> # Now sample.taco:header contains binary metadata
     """
 
     def get_schema(self) -> dict[str, pl.DataType]:
@@ -28,4 +20,4 @@ class Header(SampleExtension):
         # Extract binary header using tacotiff package
         header_bytes = tacotiff.metadata_from_tiff(str(sample.path))
 
-        return pl.DataFrame({"taco:header": [header_bytes]}, schema=self.get_schema())
+        return pl.DataFrame({"taco:header": [header_bytes]}, schema=self.get_schema())
\ No newline at end of file
