@@ -9,6 +9,7 @@ Key features:
 - Position-Isomorphic Tree (PIT) validation
 - Auto-padding for homogeneous structures
 - Efficient DataFrame construction with schema validation
+- Configurable schema strictness (strict_schema parameter)
 
 Best Practice: When mixing FOLDERs and FILEs, place all FOLDERs before FILEs
 for better organization and readability.
@@ -22,6 +23,7 @@ from typing import TYPE_CHECKING, cast
 import polars as pl
 import pydantic
 
+from tacotoolbox._column_utils import align_dataframe_schemas
 from tacotoolbox._constants import (
     METADATA_PARENT_ID,
     PADDING_PREFIX,
@@ -35,8 +37,10 @@ if TYPE_CHECKING:
 class TortillaExtension(ABC, pydantic.BaseModel):
     """Abstract base class for Tortilla extensions that add computed columns."""
 
-    return_none: bool = pydantic.Field(
-        False, description="If True, return None values while preserving schema"
+    schema_only: bool = pydantic.Field(
+        False,
+        description="If True, return None values while preserving schema",
+        validation_alias="return_none",  # Backward compatibility
     )
 
     @abstractmethod
@@ -46,21 +50,13 @@ class TortillaExtension(ABC, pydantic.BaseModel):
 
     @abstractmethod
     def _compute(self, tortilla: "Tortilla") -> pl.DataFrame:
-        """Actual computation logic - only called when return_none=False."""
+        """Actual computation logic - only called when schema_only=False."""
         pass
 
     def __call__(self, tortilla: "Tortilla") -> pl.DataFrame:
-        """
-        Process Tortilla and return computed metadata.
-
-        Args:
-            tortilla: Input Tortilla object
-
-        Returns:
-            pl.DataFrame: DataFrame with computed metadata
-        """
-        # Check return_none FIRST for performance
-        if self.return_none:
+        """Process Tortilla and return computed metadata."""
+        # Check schema_only FIRST for performance
+        if self.schema_only:
             schema = self.get_schema()
             none_data = {col_name: [None] for col_name in schema}
             return pl.DataFrame(none_data, schema=schema)
@@ -84,31 +80,23 @@ class Tortilla:
     tree structures. Padding samples have IDs starting with "__TACOPAD__" and
     preserve the schema of real samples with None values.
 
-    Example:
-        >>> # Basic tortilla
-        >>> tortilla = Tortilla(samples=[sample1, sample2, sample3])
-
-        >>> # With auto-padding
-        >>> tortilla = Tortilla(samples=my_samples, pad_to=32)
-        >>> # If my_samples has 50 items, 14 padding samples added (50 + 14 = 64)
-
-        >>> # Export metadata at different levels
-        >>> level0 = tortilla.export_metadata(deep=0)  # Current level
-        >>> level1 = tortilla.export_metadata(deep=1)  # One level deep
+    Schema Validation: By default (strict_schema=True), all samples must have
+    identical metadata schemas. Set strict_schema=False to allow heterogeneous
+    schemas with automatic None-filling for missing columns.
     """
 
-    def __init__(self, samples: list["Sample"], pad_to: int | None = None) -> None:
+    def __init__(
+        self,
+        samples: list["Sample"],
+        pad_to: int | None = None,
+        strict_schema: bool = True,
+    ) -> None:
         """
         Create Tortilla with eager DataFrame construction and schema validation.
 
-        Args:
-            samples: List of Sample objects
-            pad_to: Optional padding factor. If provided, adds dummy samples with
-                   "__TACOPAD__" prefix to make len(samples) % pad_to == 0.
-
         Raises:
-            ValueError: If samples have inconsistent metadata schemas or empty list
-                       or if Inclusive ID Rule is violated or if duplicate IDs found
+            ValueError: If samples have inconsistent metadata schemas (strict_schema=True)
+                       or empty list or if Inclusive ID Rule is violated or duplicate IDs
         """
         if not samples:
             raise ValueError("Cannot create Tortilla with empty samples list")
@@ -130,42 +118,86 @@ class Tortilla:
 
         # Extract metadata from all samples
         metadata_dfs = []
-        reference_columns = None
-
-        for i, sample in enumerate(samples):
-            sample_metadata_df = sample.export_metadata()
-
-            # Validate schema consistency
-            if reference_columns is None:
-                reference_columns = set(sample_metadata_df.columns)
-            else:
-                current_columns = set(sample_metadata_df.columns)
+        for sample in samples:
+            metadata_dfs.append(sample.export_metadata())
 
+        # Handle schema validation/alignment
+        if strict_schema:
+            # STRICT MODE: All schemas must match exactly
+            reference_columns = set(metadata_dfs[0].columns)
+            for i, df in enumerate(metadata_dfs[1:], start=1):
+                current_columns = set(df.columns)
                 if current_columns != reference_columns:
-                    missing_columns = reference_columns - current_columns
-                    extra_columns = current_columns - reference_columns
-
-                    error_msg = (
-                        f"Schema inconsistency at sample {i} (id: {sample.id}):\n"
+                    self._raise_schema_mismatch_error(
+                        i, samples[i], reference_columns, current_columns
                     )
+        else:
+            # FLEXIBLE MODE: Use helper function to align schemas
+            metadata_dfs = align_dataframe_schemas(
+                metadata_dfs, 
+                core_fields=["id", "type", "path"]
+            )
 
-                    if missing_columns:
-                        error_msg += f"  Missing columns: {sorted(missing_columns)}\n"
-
-                    if extra_columns:
-                        error_msg += f"  Extra columns: {sorted(extra_columns)}\n"
-
-                    error_msg += f"  Expected schema: {sorted(reference_columns)}\n"
-                    error_msg += f"  Actual schema: {sorted(current_columns)}"
-
-                    raise ValueError(error_msg)
-
-            metadata_dfs.append(sample_metadata_df)
-
-        # Concatenate DataFrames - all schemas are guaranteed to be consistent
+        # Concatenate DataFrames - all schemas are guaranteed consistent
         self._metadata_df = pl.concat(metadata_dfs, how="vertical")
         self._current_depth = self._calculate_current_depth()
 
+    def _raise_schema_mismatch_error(
+        self,
+        sample_index: int,
+        sample: "Sample",
+        reference_columns: set[str],
+        current_columns: set[str],
+    ) -> None:
+        """Raise helpful error when schema mismatch detected in strict mode."""
+        missing_columns = reference_columns - current_columns
+        extra_columns = current_columns - reference_columns
+
+        error_msg = (
+            f"Schema inconsistency detected at sample {sample_index} (id: '{sample.id}'):\n\n"
+        )
+
+        error_msg += (
+            f"  Reference sample columns: {sorted(reference_columns)}\n"
+            f"  Current sample columns: {sorted(current_columns)}\n\n"
+        )
+
+        if missing_columns:
+            error_msg += f"  Missing columns: {sorted(missing_columns)}\n"
+
+        if extra_columns:
+            error_msg += f"  Extra columns: {sorted(extra_columns)}\n"
+
+        error_msg += (
+            "\n"
+            "  💡 Hint: If you want to combine samples with different schemas, use:\n"
+            "      Tortilla(samples=[...], strict_schema=False)\n\n"
+            "  This will auto-fill missing columns with None values.\n\n"
+            "  To inspect sample schemas before creating Tortilla:\n"
+            "      df = sample.export_metadata()\n"
+            "      print(df.columns)  # See what columns exist\n"
+        )
+
+        raise ValueError(error_msg)
+
+    def _align_schema(
+        self, df: pl.DataFrame, target_columns: set[str]
+    ) -> pl.DataFrame:
+        """
+        Align DataFrame schema to match target columns by adding missing columns with None.
+        """
+        current_columns = set(df.columns)
+        missing_columns = target_columns - current_columns
+
+        if not missing_columns:
+            return df
+
+        # Add missing columns with None values
+        for col_name in missing_columns:
+            df = df.with_columns(pl.lit(None).alias(col_name))
+
+        return df
+
     def _validate_unique_ids(self) -> None:
         """
         Ensure all sample IDs are unique at this level.
@@ -174,9 +206,6 @@ class Tortilla:
         offsets are stored in a dictionary keyed by archive path. When duplicate
         IDs exist, later samples overwrite earlier ones in the offset map,
         resulting in missing files in the final container.
-
-        Raises:
-            ValueError: If duplicate sample IDs found with details about the duplicates
         """
         ids = [s.id for s in self.samples]
         duplicates = {id: count for id, count in Counter(ids).items() if count > 1}
@@ -204,13 +233,6 @@ class Tortilla:
         cleaned up after container creation.
 
         Padding IDs are sequential: __TACOPAD__0, __TACOPAD__1, __TACOPAD__2, etc.
-
-        Args:
-            samples: Original list of samples
-            pad_to: Target divisor for total length
-
-        Returns:
-            List with original samples plus padding samples
         """
         # Local import to avoid circular dependency
         from tacotoolbox.sample.datamodel import Sample
@@ -252,13 +274,7 @@ class Tortilla:
         return padded_samples
 
     def _check_sample_ordering(self) -> None:
-        """
-        Check if samples follow best practice: FOLDERs before FILEs.
-
-        Warns if:
-        1. There's a mix of FOLDERs and FILEs
-        2. FOLDERs appear after FILEs
-        """
+        """Check if samples follow best practice: FOLDERs before FILEs."""
         types = [sample.type for sample in self.samples]
         has_folders = "FOLDER" in types
         has_files = "FILE" in types
@@ -293,20 +309,6 @@ class Tortilla:
 
         This ensures that dataset.read("some_id") only fails if that ID
         doesn't exist in ANY folder, not just some.
-
-        Valid examples (inclusive subsets):
-            Folder A: ["img_001", "img_002", "img_003"]
-            Folder B: ["img_001", "img_002"]  # subset of A
-
-            Folder A: ["data", "mask", "thumbnail"]
-            Folder B: ["data", "mask", "__TACOPAD__0"]  # "data", "mask" subset of A
-
-        Invalid example (non-inclusive):
-            Folder A: ["img_001", "img_002"]
-            Folder B: ["img_001", "img_003"]  # "img_003" not in A, "img_002" not in B
-
-        Raises:
-            ValueError: If ID sets are not inclusive (not subset relationships)
         """
         # Collect FOLDER samples
         folder_samples = [s for s in self.samples if s.type == "FOLDER"]
@@ -396,9 +398,6 @@ class Tortilla:
         """
         Add extension data via DataFrame processing.
 
-        Args:
-            extension: Extension implementing TortillaExtension ABC
-
         Returns:
             Self for method chaining
 
@@ -429,9 +428,6 @@ class Tortilla:
                 - 0: Current level only (with extensions)
                 - >0: Expand N levels deep (base metadata only, adds internal:parent_id)
 
-        Returns:
-            DataFrame with sample metadata and optional position tracking
-
         Raises:
             ValueError: If deep is negative or exceeds maximum depth
         """
@@ -452,15 +448,6 @@ class Tortilla:
 
         Uses cumulative global index tracking to ensure parent_id values
         reference the correct row in the previous level's DataFrame.
-
-        Args:
-            target_deep: Target expansion depth
-
-        Returns:
-            DataFrame with expanded samples including internal:parent_id
-
-        Raises:
-            ValueError: If target_deep exceeds current structure depth
         """
         if target_deep > self._current_depth:
             raise ValueError(
@@ -503,4 +490,11 @@ class Tortilla:
             current_dfs = next_dfs
             current_samples = next_samples
 
+        # Align schemas before concatenating using DRY helper function        
+        if len(current_dfs) > 1:
+            current_dfs = align_dataframe_schemas(
+                current_dfs,
+                core_fields=["id", "type", "path", METADATA_PARENT_ID]
+            )
+
         return pl.concat(current_dfs, how="vertical")
\ No newline at end of file
