@@ -7,22 +7,9 @@ This module provides bidirectional conversion between TACO container formats:
 
 Key features:
 - Preserves all metadata and hierarchical structure
-- Handles AVRO colon sanitization (_COLON_ ↔ :)
 - Regenerates __meta__ files with correct offsets for ZIP
-- Supports all PIT schema patterns (FILES only, FOLDERs, mixed depths)
-
-Functions:
-    zip_to_folder(): Extract ZIP to FOLDER format
-    folder_to_zip(): Package FOLDER into ZIP format
-
-Example:
-    >>> from tacotoolbox.translate import zip_to_folder, folder_to_zip
-    >>> 
-    >>> # Convert ZIP to FOLDER
-    >>> zip_to_folder("dataset.tacozip", "dataset_folder/")
-    >>> 
-    >>> # Convert FOLDER to ZIP
-    >>> folder_to_zip("dataset_folder/", "dataset.tacozip")
+- Supports all PIT schema patterns
+- Progress bars for file operations
 """
 
 import json
@@ -31,11 +18,14 @@ from pathlib import Path
 import polars as pl
 from tacoreader import TacoDataset
 
-from tacotoolbox._constants import AVRO_COLON_REPLACEMENT
+from tacotoolbox._logging import get_logger
 from tacotoolbox._metadata import MetadataPackage
-from tacotoolbox.extract import extract
+from tacotoolbox._progress import ProgressContext
+from tacotoolbox._writers.export_writer import ExportWriter
 from tacotoolbox._writers.zip_writer import ZipWriter
 
+logger = get_logger(__name__)
+
 
 class TranslateError(Exception):
     """Raised when translation operations fail."""
@@ -46,53 +36,46 @@ class TranslateError(Exception):
 # =============================================================================
 
 
-def zip2folder(
+async def zip2folder(
     zip_path: str | Path,
     folder_output: str | Path,
-    nworkers: int = 4,
-    overwrite: bool = False,
+    concurrency: int = 100,
     quiet: bool = False,
+    debug: bool = False,
 ) -> Path:
     """
     Convert ZIP format TACO to FOLDER format.
 
-    This is a simple wrapper around extract() that converts a complete
+    This is a wrapper around ExportWriter that converts a complete
     ZIP container to FOLDER format without any filtering.
-
-    Args:
-        zip_path: Path to input .tacozip file
-        folder_output: Path to output folder
-        nworkers: Number of parallel workers for file copying
-        overwrite: If True, remove existing output folder
-        quiet: If True, suppress progress messages
-
-    Returns:
-        Path to created FOLDER container
-
-    Raises:
-        TranslateError: If conversion fails
-
-    Example:
-        >>> zip_to_folder("dataset.tacozip", "dataset_folder/")
-        PosixPath('dataset_folder')
     """
     try:
+        logger.info(f"Converting ZIP to FOLDER: {zip_path} → {folder_output}")
+
         dataset = TacoDataset(str(zip_path))
-        return extract(
+
+        writer = ExportWriter(
             dataset=dataset,
-            output=folder_output,
-            nworkers=nworkers,
-            overwrite=overwrite,
+            output=Path(folder_output),
+            concurrency=concurrency,
             quiet=quiet,
+            debug=debug,
         )
+
+        result = await writer.create_folder()
+        logger.info(f"Conversion complete: {result}")
+        return result
+
     except Exception as e:
+        logger.error(f"Failed to convert ZIP to FOLDER: {e}")
         raise TranslateError(f"Failed to convert ZIP to FOLDER: {e}") from e
 
 
 def folder2zip(
     folder_path: str | Path,
     zip_output: str | Path,
-    quiet: bool = True,
+    quiet: bool = False,
+    debug: bool = False,
     temp_dir: str | Path | None = None,
     **kwargs,
 ) -> Path:
@@ -105,53 +88,31 @@ def folder2zip(
 
     Process:
     1. Read COLLECTION.json
-    2. Read consolidated metadata (METADATA/levelX.avro)
+    2. Read consolidated metadata (METADATA/levelX.parquet)
     3. Reconstruct local_metadata from consolidated (for __meta__ generation)
     4. Scan DATA/ for physical files
     5. Use ZipWriter to create ZIP (regenerates __meta__ with offsets)
-
-    Args:
-        folder_path: Path to input FOLDER container
-        zip_output: Path to output .tacozip file
-        quiet: If True, suppress progress messages
-        temp_dir: Temporary directory for ZIP creation
-        **kwargs: Additional arguments passed to ZipWriter
-
-    Returns:
-        Path to created .tacozip file
-
-    Raises:
-        TranslateError: If conversion fails
-
-    Example:
-        >>> folder_to_zip("dataset_folder/", "dataset.tacozip")
-        PosixPath('dataset.tacozip')
     """
     folder_path = Path(folder_path)
     zip_output = Path(zip_output)
 
     try:
-        if not quiet:
-            print("Converting FOLDER to ZIP...")
+        logger.info(f"Converting FOLDER to ZIP: {folder_path} → {zip_output}")
 
         # 1. Read COLLECTION.json
-        if not quiet:
-            print("  [1/5] Reading COLLECTION.json...")
+        logger.debug("Reading COLLECTION.json")
         collection = _read_collection(folder_path)
 
-        # 2. Read consolidated metadata from METADATA/levelX.avro
-        if not quiet:
-            print("  [2/5] Reading consolidated metadata...")
+        # 2. Read consolidated metadata from METADATA/levelX.parquet
+        logger.debug("Reading consolidated metadata")
         levels = _read_consolidated_metadata(folder_path)
 
         # 3. Reconstruct local_metadata from consolidated
-        if not quiet:
-            print("  [3/5] Reconstructing local metadata...")
+        logger.debug("Reconstructing local metadata")
         local_metadata = _reconstruct_local_metadata_from_levels(levels)
 
         # 4. Scan DATA/ for physical files
-        if not quiet:
-            print("  [4/5] Scanning data files...")
+        logger.debug("Scanning data files")
         src_files, arc_files = _scan_data_files(folder_path)
 
         # 5. Create MetadataPackage
@@ -165,22 +126,24 @@ def folder2zip(
         )
 
         # 6. Use ZipWriter to create ZIP (regenerates __meta__ with offsets)
-        if not quiet:
-            print("  [5/5] Creating ZIP container...")
-        writer = ZipWriter(output_path=zip_output, quiet=quiet, temp_dir=temp_dir)
-        result = writer.create_complete_zip(
-            src_files=src_files,
-            arc_files=arc_files,
-            metadata_package=metadata_package,
-            **kwargs,
-        )
+        logger.debug("Creating ZIP container")
 
-        if not quiet:
-            print(f"Conversion complete: {result}")
+        with ProgressContext(quiet=quiet):
+            writer = ZipWriter(
+                output_path=zip_output, quiet=quiet, debug=debug, temp_dir=temp_dir
+            )
+            result = writer.create_complete_zip(
+                src_files=src_files,
+                arc_files=arc_files,
+                metadata_package=metadata_package,
+                **kwargs,
+            )
 
+        logger.info(f"Conversion complete: {result}")
         return result
 
     except Exception as e:
+        logger.error(f"Failed to convert FOLDER to ZIP: {e}")
         raise TranslateError(f"Failed to convert FOLDER to ZIP: {e}") from e
 
 
@@ -190,18 +153,7 @@ def folder2zip(
 
 
 def _read_collection(folder_path: Path) -> dict:
-    """
-    Read COLLECTION.json from FOLDER.
-
-    Args:
-        folder_path: Root path to FOLDER container
-
-    Returns:
-        Collection dictionary
-
-    Raises:
-        TranslateError: If COLLECTION.json missing or invalid
-    """
+    """Read COLLECTION.json from FOLDER."""
     collection_path = folder_path / "COLLECTION.json"
 
     if not collection_path.exists():
@@ -224,72 +176,32 @@ def _read_collection(folder_path: Path) -> dict:
 
 
 def _read_consolidated_metadata(folder_path: Path) -> list[pl.DataFrame]:
-    """
-    Read consolidated metadata from METADATA/levelX.avro files.
-
-    CRITICAL: AVRO files use _COLON_ replacement for colons in column names.
-    This function unsanitizes them back to standard format (internal:parent_id).
-
-    Args:
-        folder_path: Root path to FOLDER container
-
-    Returns:
-        List of DataFrames, one per level (sorted by level number)
-
-    Raises:
-        TranslateError: If metadata files missing or invalid
-    """
+    """Read consolidated metadata from METADATA/levelX.parquet files."""
     metadata_dir = folder_path / "METADATA"
 
     if not metadata_dir.exists():
         raise TranslateError(f"METADATA directory not found in {folder_path}")
 
-    # Find all levelX.avro files
-    level_files = sorted(metadata_dir.glob("level*.avro"))
+    # Find all levelX.parquet files
+    level_files = sorted(metadata_dir.glob("level*.parquet"))
 
     if not level_files:
-        raise TranslateError(f"No level*.avro files found in {metadata_dir}")
+        raise TranslateError(f"No level*.parquet files found in {metadata_dir}")
+
+    logger.debug(f"Found {len(level_files)} metadata files")
 
     levels = []
     for level_file in level_files:
         try:
-            df = pl.read_avro(level_file)
-            df = _unsanitize_colons(df)
+            df = pl.read_parquet(level_file)
             levels.append(df)
+            logger.debug(f"Read {level_file.name}: {len(df)} rows")
         except Exception as e:
             raise TranslateError(f"Failed to read {level_file}: {e}") from e
 
     return levels
 
 
-def _unsanitize_colons(df: pl.DataFrame) -> pl.DataFrame:
-    """
-    Convert AVRO sanitized column names back to standard format.
-
-    AVRO does not support colons in field names, so they are replaced
-    with _COLON_ during serialization. This function reverses that.
-
-    Example:
-        internal_COLON_parent_id → internal:parent_id
-        stac_COLON_crs → stac:crs
-
-    Args:
-        df: DataFrame with sanitized column names
-
-    Returns:
-        DataFrame with standard column names
-    """
-    rename_map = {
-        col: col.replace(AVRO_COLON_REPLACEMENT, ":")
-        for col in df.columns
-        if AVRO_COLON_REPLACEMENT in col
-    }
-
-    if rename_map:
-        return df.rename(rename_map)
-    return df
-
-
 def _reconstruct_local_metadata_from_levels(
     levels: list[pl.DataFrame],
 ) -> dict[str, pl.DataFrame]:
@@ -306,18 +218,6 @@ def _reconstruct_local_metadata_from_levels(
     2. Finding FOLDER samples at each level
     3. Extracting their children from the next level using internal:parent_id
     4. Removing columns not needed in __meta__ (internal:relative_path)
-
-    Args:
-        levels: List of DataFrames (consolidated metadata per level)
-
-    Returns:
-        Dictionary mapping folder_path → children DataFrame
-
-    Example:
-        >>> levels = [level0_df, level1_df, level2_df]
-        >>> local_metadata = _reconstruct_local_metadata_from_levels(levels)
-        >>> local_metadata.keys()
-        dict_keys(['DATA/folder_A/', 'DATA/folder_A/subfolder_B/', ...])
     """
     local_metadata = {}
 
@@ -353,6 +253,7 @@ def _reconstruct_local_metadata_from_levels(
 
             local_metadata[folder_path] = children
 
+    logger.debug(f"Reconstructed {len(local_metadata)} local metadata entries")
     return local_metadata
 
 
@@ -363,24 +264,6 @@ def _scan_data_files(folder_path: Path) -> tuple[list[str], list[str]]:
     Builds parallel lists of source paths (absolute filesystem paths) and
     archive paths (paths within ZIP container). Excludes __meta__ files
     since they're regenerated during ZIP creation.
-
-    Args:
-        folder_path: Root path to FOLDER container
-
-    Returns:
-        Tuple of (src_files, arc_files):
-            - src_files: List of absolute file paths
-            - arc_files: List of archive paths (e.g., "DATA/folder_A/file.tif")
-
-    Raises:
-        TranslateError: If DATA directory missing or empty
-
-    Example:
-        >>> src_files, arc_files = _scan_data_files(Path("dataset/"))
-        >>> src_files[0]
-        '/absolute/path/to/dataset/DATA/sample_001.tif'
-        >>> arc_files[0]
-        'DATA/sample_001.tif'
     """
     data_dir = folder_path / "DATA"
 
@@ -402,4 +285,5 @@ def _scan_data_files(folder_path: Path) -> tuple[list[str], list[str]]:
     if not src_files:
         raise TranslateError(f"No data files found in {data_dir}")
 
-    return src_files, arc_files
+    logger.debug(f"Scanned {len(src_files)} data files")
+    return src_files, arc_files
\ No newline at end of file
